function s = optic(varargin),

% OPTIC define a lens prescription (i.e., a series of primitives to trace 
%   through)
%
% OPTIC.  By itself, creates an empty OPTIC object.  PRIM objects are 
% added to this object.  Ray objects can be traced through an OPTIC 
% object.
% 
% The following methods can be applied to an OPTIC object 
%
% Navigation through an OPTIC object 
%
%   gohead - reset lens prescription to the start 
%   golast - set lens prescrption to the end 
%   next   - go to the next element 
%   prev   - go to the last element 
%   seek     - go to a particular PRIM
%
% Standard modification of an OPTIC object 
%
%   append  - add a new PRIM to the OPTIC object
%   set     - set a particular property of the OPTIC object
%   get     - return a particular property or PRIM object 
% 
% Reporting on an OPTIC object
%
%   length  - number of PRIM objects 
%   islast  - is the current PRIM the last one ?
%   print    - print OPTIC details  
%
% Higher level modifications  
%     
%   repo     - reposition the lens system with 1st element at the origin
%
% Additional methods to implement. 
%
%   purkinje - TO BE IMPLEMENTED
%   rev      - TO BE IMPLEMENTED 
% 
%  Examples 
% 
%   x = optic;
%   x = addon(x, 'index','air',1); 
%   x = addon(x, 'conic', 'anterior cornea', 'refract', [0 0 0], [0 0 0], 7.72, 1/7.72, -0.26); 
%   x = addon(x, 'index','cornea', n(2)); 
% 


s = init_optic;
if (nargin == 0),
    return 
end

% no additional processing implemented


return

function s = init_optic, 
s.total     = 0;
s.current   = 0;
s.optic{1}  = [];
% s.lasterr   = 0;
s = class(s, 'optic');
return 