function z = getsurf(p, x, y, varargin),

% GETSURF return surface details for the primitive. 
% (part of the RAYTRACE toolbox.)
%
% GETSURF(p, x, y, ...). Here p is the primitive, (x,y) is the position and 
% additional parameters are used to specify height or derivative. 
%
% Examples 
%
%   r0=7.5;a=0.0125;r1=1.5;r2=3.0; x = 1.5; y = 1.5;
%   p = prim('rkklein','klein-RK','on',[0 0 0],[0 10 0],4,1e-9,r0,a,r1,r2); 
%   z = getsurf(p, x, y);          % height
%   dzdx = getsurf(p, x, y, 'dzdx');  % dz/dx
%

disp('hi');

% get posn. in shfted/rotated space.
x = x - p.position(1);
y = y - p.position(2);
nOrient = (p.orientation == [ 0 0 0 ]);



if (all(nOrient)),
    % no orientation do the standard
    z = mgetfn(p.type, x, y, p.search, p.args.value{:},varargin{:});
    if (strcmp(varargin{1}, 'z')), % height requested shift
        z = z + p.position(3);
    end
    return
end

fprintf('orientation request.');

% orientation requested.
nR = getRotMtx('zyx', -p.orientation);
nX = (nR(1:2,1:2)*([ x y ]'))';
x  = nX(:,1); y  = nX(:,2);    

if (strcmp(varargin{1},'z')),
    z = mgetfn(p.type, x, y, p.search, p.args.value{:},'z');
    nX = inv(nR)*([ x y z]');
    x  = nX(:,1); y  = nX(:,2); z  = nX(:,3);
    z = z + p.position(3);
    return   
end

% requested a derivative (get normal in rotated frame)
z = mgetfn(p.type, x, y, p.search, p.args.value{:},'normal');

% rotate back to normal frame
z = inv(nR)*(z'); z = z';

switch(varargin{1}),
    case { 'dzdx' }
      z = -z(:,1)/z(:,3);
    case { 'dzdy' }
      z = -z(:,2)/z(:,3);
    case  { 'normal' }
      z(:,1) = -z(:,1)./z(:,3);
      z(:,2) = -z(:,2)./z(:,3);
      z(:,3) = -1;  
end 
return

function z = mgetfn(strFn, x, y, strType, varargin),
  % p.args.value{:}
  % p.args.value{2:end}
  switch(strType),
    case {'analytic' }
        z  = feval(strFn, x, y, varargin{:},'normal');
    case { 'numeric' }
        z  = feval(strFn, x, y, varargin{2:end},'normal');
    otherwise,
        error('(GETSURF) primitive needs a search parameter.');
  end    
end

return