function z = getsurf(p, x, y, varargin),

% GETSURF return surface details for the primitive. 
% (part of the RAYTRACE toolbox.)
%
% GETSURF(p, x, y, ...). Here p is the primitive, (x,y) is the position and 
% additional parameters are used to specify height or derivative. 
%
% Examples 
%
%   r0=7.5;a=0.0125;r1=1.5;r2=3.0; x = 1.5; y = 1.5;
%   p = prim('rkklein','klein-RK','on',[0 0 0],[0 10 0],4,1e-9,r0,a,r1,r2); 
%   p = getsurf(p, x, y);          % height
%   p = getsurf(p, x, y, 'dzdx');  % dz/dx
%

switch(p.search),
    case {'analytic' }
        z  = feval(p.type, x, y, p.args.value{:},varargin{:});
    case { 'numeric' }
        z  = feval(p.type, x, y, p.args.value{2:end},varargin{:});
    otherwise,
        error('(GETSURF) primitive needs a search parameter.');
end    
return 
