function [ Xout, Mout, Iout, eee ] = traceto_linear(X, M,  ap, C, err_tol,  II, nType),

% TRACETO_LINEAR intersection with general 
% linear surfaces.  
% 
% function [ M III ] = traceto_linear(X, M, ap, C, err_tol, I),

%    [X,I]    = traceto_linear(X, M,  myelement.aperture, myelement.X, myelement.err_tol,I,szLabel);
% 
% where [ X M ] = ray vector 
%       nType   = type of surface to intersect 'bspline'
%       C     = Information
%       M     = Intersection
%       III   = index to intersection points
% 
% also assume that the b-spline has been shifted so that 
% its centered on the origin
% 
% -------------------------------------------------------------------------
% 
% To check:
% 
% please check that the rays are Xout and Mout are the correct ones. 
%
% -------------------------------------------------------------------------


% disp('X=');
% [ X M ] 


if  ~exist('err_tol'),
    err_tol = 1.0e-5;
end

nType = 'bspline';  % HARDCODED BSPLINE

% eliminate rays outside of the aperture
s        = abs(X(:,3)./M(:,3));
s        = s(:,ones([1 3]));
X0       = M.*s + X;                         % intersection point
r        = sqrt(X0(:,1).^2 + X0(:,2).^2);    
ind      = find(r <= ap);                    % clear aperture 

if       ~isempty(ind),
          mdisp('aperture');
          X        = X(ind,:);                         % valid rays
          M        = M(ind,:);
          II       = II(ind,:);                        % indexes
end
      

% initialize
Xout = [];  % intersection points
Mout = [];
Iout = [];  % ray indexes
PP   = [];  % ray data      - mdebug('on') only
SS   = [];  % ray parameter - mdebug('on') only  
eee  = [];  % errors
acum = 0;   % accumulated work


% [ X M ]
% input('wait...');

switch(nType)
   
    case 'bspline'
        strModel = 'getbspline';
        
    case 'zernike'
        strModel  = 'zernike';
       
    otherwise
            
end

switch(nType)
  
  case 'bspline'
      
       mdisp('warning: please wait ... this may take some time');
       mdisp('warning: bspline assumed at origin'); 
       mdisp('completed ,.. 0%');
       
       % normalize P so that s is the distance along the axis
       M = M./abs([ M(:,3) M(:,3) M(:,3) ]);
         
       % initialize 
       l     = size(M,1);

       % best first guess (bounding guess SHOULD BE IMPROVED !!!) 
       s1new    =   5*ones([ l 1 ]) + X(:,3);  s1new = s1new(:);
       s2new    =  -5*ones([ l 1 ]) + X(:,3);  s2new = s2new(:);
 
       % disp('entering main...');
       k        = 1;       
       while(k < 100),
   
        s1  = s1new;     % ray parameter 
        s2  = s2new;     % thang parameter 

        % find mean position
        s   = 0.5*(s1 + s2);
       
        % error between new point surface and ray
        xtest0   = M.*[ s s s ]  + X;             % mean
        
        %
        % Model Height Field 
        %
               
        % ztest0   = getbspline(C, xtest0(:,1), xtest0(:,2) );
        ztest0  = feval(strModel,C,xtest0(:,1), xtest0(:,2)); 
                
        e0       = ztest0 - xtest0(:,3);
     
   
        temp    = abs(e0) <= err_tol;
        I       = find(temp);              % members that have terminated
        if      ~isempty(I),
                 Inot = find(~temp);
          
                 acum = acum + length(I);
                 mdisp([ 'completed ' num2str(100*(acum/l)) '  %' ]);     
            
             % accumulate the answers
             Xout  = [ Xout ; xtest0(I,1:2) ztest0(I) ];
             Mout  = [ Mout ; M(I,:) ];
          
             Iout  = [ Iout ; II(I) ];
             eee   = [ eee ; e0(I) ];
                        
             % full([ Xout Iout ])
             % input('extractor');
             
             if  mdebug,
                 PP      = [ PP  ; X(I,:) M(I,:) ];
                 SS      = [ SS  ; s(I) ];
             end
    
             
            if      isempty(Inot),
                 
                    % full([ Xout Iout ])             
                    % input('wait mon...');       
                    mdisp('done.');
                
                    [ Iout I ] = sort(Iout);
                    Xout       = Xout(I,:);
                    Mout       = Mout(I,:);
                    
                    % Iout       = Iout(I,:);
                    % input('extractor');
                    eee        = eee(I,:);

                
                    if  mdebug, 
                        disp('objective error report');
                        PP = PP(I,:);
                        SS = SS(I);
                               
                        %
                        % Model Height Field 
                        %
                        
                        % ztest1 = getbspline(C, Xout(:,1), Xout(:,2));
                        ztest1  = feval(strModel,C,Xout(:,1), Xout(:,2)); 
                        
                        
                        ztest2 = PP(:,6).*SS + PP(:,3);
                        full([ ztest1(:) ztest2(:) ])
                        er     = abs(ztest1 - ztest2);
                        disp([ 'objective error mean/max  :' num2str(mean(er)) ' ' num2str(max(er)) ]);
                  
                        Iout
                        Mout
                        Xout    
                          
                        input('wait...');  
                    end
                    
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    % here we go. 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                                       
                    return
            end
             
            s1      = s1(Inot);
            s       = s(Inot);
            s2      = s2(Inot);
            X       = X(Inot, :);
            M       = M(Inot, :);
            II      = II(Inot);
            
            ztest0  = ztest0(Inot, :);
            xtest0  = xtest0(Inot, :);
            e0      = e0(Inot,:);
            % e1      = e1(Inot,:);      
            Inot    = [];
         end
     
         
     % ray points and cooresponding b-spline height
     xtest1    = M.*[ s1 s1 s1 ] + X;                % point
     
     % ztest1    = getbspline(C, xtest1(:,1), xtest1(:,2) );
     ztest1  = feval(strModel,C,xtest1(:,1), xtest1(:,2));    
     e1        = ztest1 - xtest1(:,3);      
 
     % subdivide the remaining rays  
     temp  = (e0.*e1 <= 0);
     i1    = find(temp);        % s1 -> s
     i2    = find(~temp);       % s  -> s2
     
     % problem must be here!!
     s1new = [ s1(i1) ; s(i2) ];  s2new = [ s(i1) ; s2(i2) ];
     
     % I think P has to be re-arranged as well
     X     = [ X(i1,:) ; X(i2,:) ]; 
     M     = [ M(i1,:) ; M(i2,:) ];  
     II    = [ II(i1,:) ; II(i2,:) ];
     k     = k + 1;

     if ~mod(k,20)
         disp([ 'Iterations ' num2str(k) ]);
     end   
       
       end

       disp('warning: partial termination only'); 
  
  end
     
  
    
  
  
return 




function ztest0 = getbspline(X, x, y);

 % [ x y ]
 ztest0 = bsp_2d( X, x, y );
 
return



function ztest0 = getzernike(X, x, y);

 % use different methods depending on number of rays
 l = length(x);
 
 % [ x y ]
 if   ( l < 2),
      ztest0 = bspnp2d( X, x, y );
 else 
      ztest0 = bspgetbig( X, x, y );
 end


return
