function [s, II] = traceto_quadric(s,info,II),

% TRACETO_QUADRIC intersect a quadric surface
% with a ray, where the surface is origin centered,
% 
% [X, M, II] = traceto_toroid(X,M,info,II),
% 
% where X               = ray position
%       M               = ray slope
%       info.Q          = aspericity parameter
%       info.R          = radius of curvature
%       info.aperture   = aperture (radius of aperture).
% 
% nb: solve_quartic needs root solving fix.
% use MATLAB root equation.

% disp('RAYS');
% get(s,'posn')
% pause;

X0      = get(s,'posn');
M       = get(s,'slope');
OPL     = get(s,'OPL');
nindex  = get(s,'index');

% X0
% pause;

% info    = [ 0 0 0 ap 0 ];

ndx         = (1:size(X0,1))';  % internal index

[X,M,ndx]   = m_traceto_quadric(X0, M, info, ndx);

II          = II(ndx);
OPL         = OPL(ndx);
X0          = X0(ndx,:);

if (isempty(II)),
    
        disp('jumping out this way');        
        s = set(s,'posn',[]);
        s = set(s,'slope',[]);
        s = set(s,'OPL',[]);
        return 
end


dz  = X(:,3) - X0(:,3);
ds  = M(:,3);
new_OPL     = nindex*sign(dz.*ds).*vec_norm(X-X0) + OPL;
    


s = set(s,'posn',X);
s = set(s,'slope',M);
s = set(s,'OPL',new_OPL);
return


% M_TRACETO_QUADRIC trace to quadric surface
% 
% 

function [X,M,II] = m_traceto_quadric(X, M, info, ndx),

 
if  ~exist('II'),
    II = 1:size(M,1); 
    II = II(:);
end

Q   = info.Q;           % x-radius of xcurvature
R   = -info.R;          % y-radius of curvature  (4/12/04 added -ve )
ap  = info.aperture;    

% switch(info.sense)
%
%    case 'concave'
%        f0 = -1;
%
%    case 'convex'
%        f0 = 1;
%
%    otherwise
%        info.sense 
%        error('traceto_quadric: unknown sense');
%               
% end


u = M(:,1);
v = M(:,2);
w = M(:,3);

u0 = X(:,1);
v0 = X(:,2);
w0 = X(:,3);

% ------------------------------------------------------------------------------
%
% intersection equation
%  
%   a0.*t.^2+b0.*t+c0 = 0
% 
% where 
%  
    a0 = u.^2 + v.^2 + (1+Q)*w.^2;
    b0 = 2*u.*u0 + 2*v.*v0 + 2*(1+Q)*w.*w0 + 2*R*w;
    c0 = u0.^2 + v0.^2 + (1+Q)*w0.^2 + 2*R*w0;
% 
%   nb: this is the solution to the -ve quadric
%   this is the standard solution to the normal (concave up) quadric
%  
%    a0 = u.^2 + v.^2 + (1+Q)*w.^2;
%    b0 = 2*u.*u0 + 2*v.*v0 + 2*(1+Q)*w.*w0 - 2*R*w;
%    c0 = u0.^2 + v0.^2 + (1+Q)*w0.^2 - 2*R*w0;
%
% ------------------------------------------------------------------------------

% ---------------------------------
% slow root finder - but correct !!
% ---------------------------------
for k = 1:size(a0,1),
    temp    = roots([a0(k) b0(k) c0(k) ]);
    t(k,:)  = temp(:)';
end

% -----------------------------------------
% 
% choose intersection closest to the origin
% 
% ------------------------------------------

p  = [ u v w ].*[ t(:,1) t(:,1) t(:,1) ] + [ u0 v0 w0 ]; e(1,:)  = sum((p.*p)');
p  = [ u v w ].*[ t(:,2) t(:,2) t(:,2) ] + [ u0 v0 w0 ]; e(2,:)  = sum((p.*p)');
% p  = [];

[emin endx ] = min(e);
ndx          = sub2ind([ size(t,1)  4 ], 1:1:length(endx), endx);
t            = t(ndx);
t            = t(:);
X            = [ u v w].*[ t t t ] + [ u0 v0 w0 ];

% complies with 

% z_test = quadric(X(:,1),X(:,2),info);
% format long
% X(:,1) 
% X(:,2)
% X(:,3)
% z_test(:)
% pause;


% ------------------------------------------
% filter out through aperture
% ------------------------------------------

r           = sqrt( X(:,1).^2 + X(:,2).^2 );
ndx         = find(r < ap);

if          ~isempty(ndx),
            X = X(ndx,:);
            M = M(ndx,:);
            II = II(ndx);
            return
end

% -----------------------------
% no rays made through aperture
% -----------------------------

X = [];
M = [];
II = [];
 
return
