function [X, M, II] = traceto_toroid(X,M,info,II),

% INTERSECT_TORIC intersect a toric surface
% with a ray, where the surface is origin centered,
% 
% [X, M, II] = traceto_toroid(X,M,info,II),
% 
% where X               = ray position
%       M               = ray slope
%       info.rh         = horizontal radius
%       info.rv         = vertical radius
%       info.aperture   = aperture
% 
% nb: solve_quartic needs root solving fix.
% use MATLAB root equation.
% 
% NOT MODIFIED FOR raytrak-v2.3

 
if  ~exist('II'),
    II = 1:size(M,1); 
    II = II(:);
end

rh  = abs(info.rh);           % x-radius of xcurvature
rv  = abs(info.rv);           % y-radius of curvature
ap  = info.aperture;    

switch(info.sense)

    case 'concave'
        f0 = -1;

    case 'convex'
        f0 = 1;

    otherwise
        info.sense 
        error('traceto_toric: unknown sense');
               
end

rh = f0*rh;
rv = f0*rv;




u = M(:,1);
v = M(:,2);
w = M(:,3);

u0 = X(:,1);
v0 = X(:,2);
w0 = X(:,3);

% ------------------------------------------------------------------------------
%
% intersection equation
%  
%   a0.*t.^4+b0.*t.^3+c0.*t.^2+d0.*t+e0= 0
% 
% where 
%  
    a0 = (w.^4+u.^4+2.*u.^2.*w.^2+2.*v.^2.*u.^2+2.*v.^2.*w.^2+v.^4);
    b0 = (4.*u.^3.*u0+4.*u.^2.*w.*rh+4.*v.^2.*u.*u0+4.*v.^2.*w.*w0+4.*v.*v0.*w.^2+4.*w.^3.*w0+4.*u.^2.*w.*w0+4.*v.^2.*w.*rh+4.*w.^3.*rh+4.*u.*u0.*w.^2+4.*v.^3.*v0+4.*v.*v0.*u.^2);
    c0 = (8.*u.*u0.*w.*rh-4.*rv.^2.*u.^2+8.*v.*v0.*w.*w0+4.*rh.^2.*v.^2+12.*w.^2.*w0.*rh+4.*v.^2.*w0.*rh-4.*rv.^2.*w.^2+6.*w.^2.*w0.^2+4.*w.^2.*rh.^2+6.*v.^2.*v0.^2+6.*u.^2.*u0.^2+4.*rh.*rv.*w.^2+4.*rh.*rv.*u.^2-4.*rh.*rv.*v.^2+4.*u.^2.*w0.*rh+2.*u.^2.*w0.^2+2.*v0.^2.*w.^2+8.*u.*u0.*w.*w0+8.*v.*v0.*u.*u0+2.*v0.^2.*u.^2+2.*u0.^2.*w.^2+8.*v.*v0.*w.*rh+2.*v.^2.*u0.^2+2.*v.^2.*w0.^2);
    d0 = (4.*u0.^2.*w.*w0+4.*u.*u0.^3+4.*v0.^2.*u.*u0+8.*rh.*rv.*u.*u0+8.*u.*u0.*w0.*rh+4.*w.*w0.^3+8.*rh.*rv.*w.*w0-8.*rv.^2.*w.*w0-8.*rv.^2.*u.*u0+8.*rh.^2.*rv.*w+8.*v.*v0.*w0.*rh+4.*u0.^2.*w.*rh+8.*rh.^2.*v.*v0+4.*v.*v0.*u0.^2-8.*rv.^2.*w.*rh+4.*v.*v0.*w0.^2+4.*u.*u0.*w0.^2+4.*v.*v0.^3-8.*rh.*rv.*v.*v0+4.*v0.^2.*w.*w0+4.*v0.^2.*w.*rh+12.*w.*w0.^2.*rh+8.*w.*rh.^2.*w0);
    e0 = -4.*rv.^2.*w0.^2+4.*v0.^2.*w0.*rh-8.*rv.^2.*w0.*rh+8.*rh.^2.*rv.*w0+4.*rh.*rv.*w0.^2+4.*rh.*rv.*u0.^2-4.*rv.^2.*u0.^2+4.*rh.^2.*v0.^2+4.*u0.^2.*w0.*rh+2.*u0.^2.*w0.^2+u0.^4+2.*v0.^2.*u0.^2-4.*rh.*rv.*v0.^2+w0.^4+4.*w0.^3.*rh+v0.^4+2.*v0.^2.*w0.^2+4.*w0.^2.*rh.^2;
% 
% ------------------------------------------------------------------------------

% this needs fixing!!!
% 
% the roots are all jacked up .... why?!
%   t  = solve_quartic(a0,b0,c0,d0,e0);
% 
% there is a bug in solve quartic


% ---------------------------------
% slow root finder - but correct !!
% ---------------------------------
for k = 1:size(a0,1),
    temp    = roots([a0(k) b0(k) c0(k) d0(k) e0(k) ]);
    t(k,:)  = temp(:)';
end

% -----------------------------------------
% 
% choose intersection closest to the origin
% 
% ------------------------------------------

p  = [ u v w ].*[ t(:,1) t(:,1) t(:,1) ] + [ u0 v0 w0 ]; e(1,:)  = sum((p.*p)');
p  = [ u v w ].*[ t(:,2) t(:,2) t(:,2) ] + [ u0 v0 w0 ]; e(2,:)  = sum((p.*p)');
p  = [ u v w ].*[ t(:,3) t(:,3) t(:,3) ] + [ u0 v0 w0 ]; e(3,:)  = sum((p.*p)');
p  = [ u v w ].*[ t(:,4) t(:,4) t(:,4) ] + [ u0 v0 w0 ]; e(4,:)  = sum((p.*p)');
p  = [];

[emin endx ] = min(e);
ndx          = sub2ind([ size(t,1)  4 ], 1:1:length(endx), endx);
t            = t(ndx);
t            = t(:);
X            = [ u v w].*[ t t t ] + [ u0 v0 w0 ];

% ------------------------------------------
% filter out through aperture
% ------------------------------------------

r           = sqrt( X(:,1).^2 + X(:,2).^2 );
ndx         = find(r < ap);

if          ~isempty(ndx),
            X = X(ndx,:);
            M = M(ndx,:);
            II = II(ndx);
            return
end


% -----------------------------
% no rays made through aperture
% -----------------------------

X = [];
M = [];
II = [];
 
return
