% DEMO2 Demonstrate printing/plotting features and pupil function
%

clc; home;
close all;
clear all;


s0 = -6000;
nLambda0 = 0.5893e-3;     
nApEntr = 2.25;
nApID = 6;   
nD    = 4;
nGrid = 128;

disp('Setup some variables :');

fprintf('\nAxial source distance (s0)         = %2.6f\n', s0);
fprintf('Vacuum wavelength (nLambda0)       =  %2.6f\n', nLambda0);
fprintf('Entrance Beam Aperture (nApEntr)   =  %2.6f\n', nApEntr);
fprintf('Lens ID of Aperture Stop (nApID)   =  %2.6f\n', nApID);
fprintf('Pupil Diameter at Aperture Stop    =  %2.6f\n', nD);
fprintf('Pupil Sampling Grid (nGrid)        =  %2d\n', nGrid);

fprintf('\n\n');

disp('Press any key to continue ...');
pause;

fprintf('\n');
disp('Set-up an eye model with the image plane at the best focal plane, i.e.,');
disp('as determined by the plane of minimum RMS OPD. To calculate that plane,');  
disp('the paraxial exit pupil plane (nApZ) is the apex of the ref. sphere  :');
fprintf('\n'); 
disp('p    = eye_navarro;'); 
p = eye_navarro;  
disp('nApZ = getPupil(p, nApID, ''exit'')  % Axial Position of the Exit pupil'); 
nApZ = getPupil(p, nApID, 'exit')
disp('nFocImg = focus(p, s0, nAp, ''minRMSOPD'', nGrid, nApZ)  % Plane of min. RMS OPD'); 
nFocImg = focus(p, s0, nApEntr, 'minRMSOPD', nGrid, nApZ)

disp('p = eye_navarro(''PupilDiameter'', nD, ''FocalLength'', nFocImg);');
p = eye_navarro('PupilDiameter', nD, 'FocalLength', nFocImg);

fprintf('\n');
disp('Press any key to continue ...');
pause;

clc;
home;

disp('Display the adjusted eye model :');
fprintf('\n');
fprintf('figure(1); clf;\n');
fprintf('plot_lens(p);\n');
fprintf('plot_ray(p);\n');

figure(1); clf;
plot_lens(p);
plot_ray(p);

fprintf('\n');
disp('Press any key to continue ...');
pause;
clc; home;

disp('Get the aperture fn. (w, pupil) : ');
fprintf('\n');

disp('nIndex  = get(seek(p, 9),''arg'',''index'')       % Image space ref. index.');
nIndex  = get(seek(p, 9),'args','index')
fprintf('Ref. Index  (nIndex)    = %2.6e\n', nIndex);
fprintf('\n');

disp('nLambda  = nLambda0/nIndex;');
nLambda  = nLambda0/nIndex;

% actual aperture
disp('[w, pupil, nD] = pfn(p, s0, nGrid, nApEntr, nApZ, nLambda); % (w, pupil) aperture fn.'); 
[w, pupil, nD]     = pfn(p, s0, nGrid, nApEntr, nApZ, nLambda0);  % w in waves 

fprintf('\n');

disp('[ nb : nD is the estimated (non-paraxial) diameter of the image of the aperture stop.]'); 

fprintf('\n');
disp('and plot it :');
fprintf('\n');

disp('figure(2); clf;'); figure(2); clf;
disp('plot_pfn(w, pupil);');
plot_pfn(w, pupil);
fprintf('\n');
disp('Press any key to continue ...');

pause;
clc; home;

disp('Calculate some scaling factors:');
fprintf('\n');

nR     = getRefSph(p, s0, nApZ); % ref. sph. 
M      = size(w,1);
N      = 2*M;
dY1    = nD/M;                  % exit pupil cell size
Fs1    = 1/dY1;                 % Sampling rate 
Fc1    = 1/(2*dY1);             % Cutoff freq. (exit pupil) 
dY2    = (nLambda*nR)/(N*dY1);  % image plane cell size  

fprintf('dY2             = %2.6e mm\n', dY2);
fprintf('Source          = %2.6e mm\n', s0);
fprintf('Pupil. Diameter = %2.6e mm\n', nD);
fprintf('Ref. Sph.       = %2.6e mm\n', nR);
fprintf('\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PSF

disp('Plot the Diffraction Point Spread Function (PSF) : ');
fprintf('\n');
disp('figure(3); clf;');
disp('[z, x, y] = pfn2psf(w, pupil, nD, nR, nLambda, nIndex);');
disp('mesh(x, y, z);');
figure(3); clf;
[z, x, y] = pfn2psf(w, pupil, nD, nR, nLambda, nIndex);
mesh(x, y, z);
fprintf('\n');
disp('Plot the Diffraction Modulation Transfer Function (DMTF) alongside the ');
disp('MTF for the perfect wavefront : ');
fprintf('\n');
disp('figure(4); clf;');
disp('[f, fx, fy] = pfn2dmtf(w, pupil, nD, nR, nLambda, nIndex);');
disp('p0 = double(pupil);');
disp('[f_airy, fx_airy] = pfn2dmtf(p0, p0, nD, nR, nLambda, nIndex);');
disp('h = plot(fx(1,:), f(1,:), ''b-'', fx_airy(1,:), f_airy(1,:), ''r-'');'); 
disp('legend(h, ''Navarro'',''Diff. Limited.'');');
disp('title(''Navarro Eye MTF'');');

figure(4); clf;
[f, fx, fy] = pfn2dmtf(w, pupil, nD, nR, nLambda, nIndex);
p0 = double(pupil);
[f_airy, fx_airy] = pfn2dmtf(p0, p0, nD, nR, nLambda, nIndex);
h = plot(fx(1,:), f(1,:), 'b-', fx_airy(1,:), f_airy(1,:), 'r-'); 
legend(h, 'Navarro','Diff. Limited.');
title('Navarro Eye MTF');

return