function h = plot_lens(p, varargin),

% PLOT_LENS plot the lens prescription 
%
% PLOT_LENS(p).  Plot the lens prescription where p is the lens
% prescription object.
%
% Example 
%
%   x = eye_navarro;
%   plot_ray(x);
%   plot_lens(x);
% 

set(gcf,'DefaultLineLineWidth',1.0)

nApBig = 0;
isAp = false; 
isFirst = true;
if (nargin > 1),
    nAp = getProperty('Apertures', varargin, []);
    if (~isempty(nAp)), 
        isAp = true;
    end
end


p = gohead(p);
while(~islast(p)),

    p = next(p);
    
    nPr     = get(p);
    nStr    = get(nPr, 'type');
    if (~isAp),
        nAp     = get(nPr, 'aperture');
    end
        
    if (~isfinite(nAp)),
        nAp = 10;
    end
    
    nShift  = get(nPr, 'position');
    nOrient = get(nPr, 'orientation');
        
    % horizontal
    nR    = getRotMtx('zyx', degtorad(nOrient));  
    nY    = linspace(-nAp, nAp, 64)'; nX  = 0*nY;
    nArgs = get(nPr,'args');
    nZ    = feval(nStr, nX, nY, nArgs{:});
    
    N  = nR*([ nX nY nZ ])'; N = N';
    N(:,1) = N(:,1) + nShift(1);
    N(:,2) = N(:,2) + nShift(2);
    N(:,3) = N(:,3) + nShift(3);
    
    nX = N(:,1); nY = N(:,2); nZ = N(:,3);
    line(nZ, nY);
    
    if (~isFirst),
        line([ nZ1 nZ(1) ]', [ nY1 nY(1) ]');
        line([ nZ2 nZ(end) ]', [ nY2 nY(end) ]');
    end
    isFirst = false;

    nZ1 = nZ(1); nZ2 = nZ(end);  % top points
    nY1 = nY(1); nY2 = nY(end);  % last points
       
    
    nApBig = max([nApBig ; nY(:) ]);
    
    p = next(p);
end

% 2-D rescaling
x = axis; a0 = 0.1*(x(2)-x(1));
axis([ x(1)-a0, x(2)+a0, -1.75*nApBig, 1.75*nApBig  ]);
grid on;
return