function h = plot_ray(s, varargin),

% PLOT_RAY plot rays through the lens system.
%
% PLOT_RAY(p, r).  Plot the rays (r) through lens prescription (p).
%
% PLOT_RAY(p). Plot the lens prescription (p) using default rays. 
%
% Example,  
%
%   x = eye_navarro;
%   plot_lens(x);
%   plot_ray(x);
% 

if (nargin == 2),
    if (isa(varargin{1},'ray')),
        r = varargin{1};
    elseif (isnumeric(varargin{1})),
        %automatic
        s       = seek(s, 2); npr = get(s);
        nAp     = varargin{1};
        nPos(:,2) = linspace(-nAp, nAp, 8);
        nPos(:,1) = 0;
        nPos(:,3) = -1e3;
        nSlope = [ 0 0 1];
        r = ray(nPos,nSlope);        
    else 
        error('(PLOT_RAYS) Unknown invocation.');
    end
else
    %automatic
    s       = seek(s, 2); npr = get(s);
    nAp     = 0.95*get(npr, 'aperture');    
   
    nPos(:,2) = linspace(-nAp, nAp, 16)';
    nPos(:,1) = 0;
    nPos(:,3) = -1e3;
    nSlope = [ 0 0 1];
    r = ray(nPos,nSlope);
end 

% initialize
n1      = 1; 
n2      = 1;

% rewind optics
s = gohead(s);
n = 1:length(r);

% main
while   (~islast(s)),
   r0 = r; j = 1:length(r0);
   [r, s, x] = step(r, s);
   j = j(x.indexes); 

   nX1 = get(r0, 'position'); nX1 = nX1(j, :);
   nX2 = get(r, 'position');
      
   % connect remaining to new ones
   h = line([ nX1(:,3)' ; nX2(:,3)' ], [ nX1(:,2)' ; nX2(:,2)' ]);
   set(h,'color',[1 0 0]);
   
   
end

return