function pa = powera(nk, r, dfdr),

% POWERA Axial power given by, 
%
%           P = (nk-1)/da 
%   where 
%           1/da = (dfdr)/(r*sqrt(1+(dfdr)^2))
%
% POWERA(nk, r, dfdr).  Here nk is the refractive index of the cornea,
% r is the radial distance to the corneal point, dfdr is the slope. 
%
% Example, 
%
%   r = 1:4;  R = 7;  nk = 1.3375; % or 1.376
%   dfdr  = sphere(0, r, R, 'dzdy');
%   P     = powera(nk, r, dfdr)
%   R_est = ((nk-1)./P) 

da = (r.*sqrt(1+(dfdr).^2))./(dfdr);
pa = (nk-1)./da;

