function [r, n, x] = trace(r, s),

% TRACE trace rays (r) through entire optical system (s). 
% 
% [r, n] = TRACE(r, s). Here r is the ray object, and s is the 
% optics object.  This function returns the indexes (n) into the  
% last surface. 
% 
% [r, n, x] = TRACE(r, s). Here r is the ray object, and s is the 
% optics object.  This function returns a index into the rays object (m) 
% as well as a structure (x) determined by the last surface.  
% 

global glbTrace;

% --------------------------------------------------
% notes:
% Bug idetified 20/10/04 - in toric (doesnt like rh=rv)
% Tues 9th Oct 2001 -  optic decenter not supported.
% Dec. 5th 2001     -  optic decenter implemented.
% ---------------------------------------------------

if (~isa(r, 'ray')|~isa(s, 'optic')),
    error('(TRACE) Correct invocation is TRACE(ray, optic)');
end

% initialize
n1      = 1; 
n2      = 1;

% rewind optics
s = gohead(s);

n = 1:length(r);

% main
while   (~islast(s)),
   
   r0 = r;
   [r, s, x] = step(r, s);
   
      % logging
   if (glbTrace), 
      fprintf('-------------------------------------------------------\n');
      fprintf('INDEXES (n1 = %2.6f, n2 = %2.6f)\n', x.n1, x.n2); 
      print(x.prim);
      fprintf('RAYS\n');
      na = length(n);
      fprintf('  INPUT RAYS   = %d\n', na); 
   end
   
   n = n(x.indexes); 
   
  if (glbTrace), 
      nb = length(n);
      fprintf('  OUTPUT RAYS  = %d\n', nb);

      fprintf('  BLOCKED RAYS = %d\n', na-nb);   
      
%      fprintf('INPUT RAY.\n');
%      print(r0);
%      fprintf('OUTPUT RAY.\n');
%      print(r);
   end

end

return
